"""Test setup for integration and functional tests.

When we import PloneTestCase and then call setupPloneSite(), all of
Plone's products are loaded, and a Plone site will be created. This
happens at module level, which makes it faster to run each test, but
slows down test runner startup.
"""
import re
import transaction
from zope.component import getUtility

# Starting from plone.memoize v1.1a4 (used in plone4), global ram cache
# utility provides other IRAMCache interface, than before
try:
    # In plone4 provides
    from zope.ramcache.interfaces.ram import IRAMCache
except ImportError:
    # In plone3 provides
    from zope.app.cache.interfaces.ram import IRAMCache

from Products.Five import zcml
from Products.Five import fiveconfigure

from Testing import ZopeTestCase as ztc

from Products.CMFCore.utils import getToolByName

from Products.PloneTestCase.layer import PloneSite
from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase import setup as ptc_setup

from Products.PloneTestCase.PloneTestCase import portal_owner
from Products.PloneTestCase.PloneTestCase import default_password

from quintagroup.seoptimizer.config import PROJECT_NAME
from quintagroup.seoptimizer.config import SUPPORT_BLAYER

ptc.setupPloneSite()

class NotInstalled(PloneSite):
    """ Only package register, without installation into portal
    """

    @classmethod
    def setUp(cls):
        fiveconfigure.debug_mode = True
        import quintagroup.seoptimizer
        zcml.load_config('configure.zcml', quintagroup.seoptimizer)
        fiveconfigure.debug_mode = False
        ztc.installPackage(PROJECT_NAME)


class Installed(NotInstalled):
    """ Install product into the portal
    """
    @classmethod
    def setUp(cls):
        app = ztc.app()
        portal = app[ptc_setup.portal_name]

        # Sets the local site/manager
        ptc_setup._placefulSetUp(portal)
        # Install PROJECT
        qi = getattr(portal, 'portal_quickinstaller', None)
        qi.installProduct(PROJECT_NAME)
        transaction.commit()

    @classmethod
    def tearDown(cls):
        ptc_setup._placefulTearDown()
        

class MixinTestCase:

    def _getauth(self):
        # Fix authenticator for the form
        import re
        try:
            authenticator = self.portal.restrictedTraverse("@@authenticator")
        except:
            handle  = ""
        else:
            html = authenticator.authenticator()
            handle = re.search('value="(.*)"', html).groups()[0]
        return handle

    def beforeTearDown(self):
        getUtility(IRAMCache).invalidateAll()


class TestCase(MixinTestCase, ptc.PloneTestCase):
    layer = Installed

class TestCaseNotInstalled(MixinTestCase, ptc.PloneTestCase):
    layer = NotInstalled


class FunctionalTestCase(MixinTestCase, ptc.FunctionalTestCase):
    layer = Installed

class FunctionalTestCaseNotInstalled(MixinTestCase, ptc.FunctionalTestCase):
    layer = NotInstalled



from AccessControl import allow_module
from zope.i18nmessageid import MessageFactory

SeoptimizerMessageFactory = MessageFactory('quintagroup.seoptimizer')

allow_module('quintagroup.seoptimizer.util')



import re

from zope.interface import implements
from zope.component import queryAdapter
from zope.component import queryMultiAdapter

from quintagroup.seoptimizer.util import SortedDict
from quintagroup.seoptimizer.interfaces import IMetaKeywords, IMappingMetaTags
from quintagroup.seoptimizer.browser.seo_configlet import ISEOConfigletSchema

METADATA_MAPS = dict([
    ("DC.publisher", "Publisher"),
    ("DC.description", "Description"),
    ("DC.contributors", "Contributors"),
    ("DC.creator", "Creator"),
    ("DC.format", "Format"),
    ("DC.rights", "Rights"),
    ("DC.language", "Language"),
    ("DC.date.modified", "ModificationDate"),
    ("DC.date.created", "CreationDate"),
    ("DC.type", "Type"),
    ("DC.subject", "Subject"),
    ("DC.distribution", "seo_distribution"),
    ("description", "seo_description"),
    ("keywords", "meta_keywords"),
    ("robots", "seo_robots"),
    ("distribution", "seo_distribution")])

class MetaKeywordsAdapter(object):
    implements(IMetaKeywords)

    def __init__(self, context):
        self.context = context

    def getMetaKeywords(self):
        """ See interface.
        """
        meta_keywords = []
        seo_context = queryMultiAdapter((self.context, self.context.REQUEST),
                                        name='seo_context')
        if seo_context:
            meta_keywords = list(seo_context['meta_keywords'])
        return ', '.join(meta_keywords)


class MappingMetaTags(object):
    implements(IMappingMetaTags)

    def __init__(self, context):
        self.context = context
        pps = queryMultiAdapter((self.context, self.context.REQUEST),
                                name="plone_portal_state")
        self.gseo = queryAdapter(pps.portal(), ISEOConfigletSchema)

    def getMappingMetaTags(self):
        """ See interface.
        """
        metadata_name = SortedDict()
        if self.gseo:
            for mt in self.gseo.metatags_order:
                if METADATA_MAPS.has_key(mt):
                    metadata_name[mt] = METADATA_MAPS[mt]
        return metadata_name



from quintagroup.canonicalpath.interfaces import ICanonicalLink
from base import *

class TestCanonicalURL(FunctionalTestCase):

    def afterSetUp(self):
        self.basic_auth = ':'.join((portal_owner,default_password))
        self.loginAsPortalOwner()
        # Preparation for functional testing
        self.portal.invokeFactory('Document', id='mydoc')
        self.mydoc = self.portal['mydoc']
        self.mydoc_path = "/%s" % self.mydoc.absolute_url(1)
        self.curl = re.compile('<link\srel\s*=\s*"canonical"\s+' \
            '[^>]*href\s*=\s*\"([^\"]*)\"[^>]*>', re.S|re.M)

    def test_CanonicalURL(self):
        html = self.publish(self.mydoc_path, self.basic_auth).getBody()
        foundcurls = self.curl.findall(html)
        mydoc_url = self.mydoc.absolute_url()

        self.assertTrue([1 for curl in foundcurls if curl==mydoc_url],
           "Wrong CANONICAL URL for document: %s, all must be: %s" % (
           foundcurls, mydoc_url))

    def test_updateCanonicalURL(self):
        mydoc_url_new = self.mydoc.absolute_url() + '.new'
        # Update canonical url property
        self.publish(self.mydoc_path + '/@@seo-context-properties?' \
           'seo_canonical_override=checked&seo_canonical=%s&' \
           'form.submitted=1' % mydoc_url_new, self.basic_auth)
        # Test updated canonical url
        html = self.publish(self.mydoc_path, self.basic_auth).getBody()
        foundcurls = self.curl.findall(html)

        qseo_url = ICanonicalLink(self.mydoc).canonical_link
        self.assertTrue(qseo_url == mydoc_url_new,
           "Not set 'qSEO_canonical' property")
        self.assertTrue([1 for curl in foundcurls if curl==mydoc_url_new],
           "Wrong CANONICAL URL for document: %s, all must be: %s" % (
           foundcurls, mydoc_url_new))

    def test_defaultCanonical(self):
        expect = self.mydoc.absolute_url()
        cpath = ICanonicalLink(self.mydoc).canonical_link
        self.assertTrue(cpath == expect,
            "Default canonical link adapter return: '%s', must be: '%s'" % (
             cpath, expect))

    def testCatalogUpdated(self):
        purl = getToolByName(self.portal, 'portal_url')
        catalog = getToolByName(self.portal, 'portal_catalog')
        catalog.addColumn('canonical_link')

        # get catalog data before update
        mydoc_catalog_canonical = catalog(id="mydoc")[0].canonical_link
        self.assertTrue(not mydoc_catalog_canonical)

        # Update canonical url property
        mydoc_url_new = self.mydoc.absolute_url() + '.new'
        self.publish(self.mydoc_path + '/@@seo-context-properties?' \
            'seo_canonical_override=checked&seo_canonical=%s' \
            '&form.submitted=1' % mydoc_url_new, self.basic_auth)

        newcpath = ICanonicalLink(self.mydoc).canonical_link
        mydoc_catalog_canonical = catalog(id="mydoc")[0].canonical_link
        self.assertTrue(newcpath == mydoc_catalog_canonical,
            "canonical path get by adapter: '%s' not equals to cataloged one: '%s'" % (
             newcpath, mydoc_catalog_canonical))

    def test_canonicalValidation(self):
        wrong_canonical = 'wrong   canonical'
        # Update canonical url property
        html = self.publish(self.mydoc_path + '/@@seo-context-properties?' \
               'seo_canonical_override=checked&seo_canonical=%s&' \
               'form.submitted=1' % wrong_canonical, self.basic_auth).getBody()
        self.assertTrue("wrong canonical url" in html,
                        "Canonical url not validated")

    def test_delCanonical(self):
        newcanonical = '/new_canonical'
        ICanonicalLink(self.mydoc).canonical_link = newcanonical

        assert ICanonicalLink(self.mydoc).canonical_link == newcanonical

        # remove canonical url customization
        self.publish(self.mydoc_path + '/@@seo-context-properties?' \
           'seo_canonical=%s&seo_canonical_override=&form.submitted=1' % newcanonical,
            self.basic_auth)

        mydoc_canonical = ICanonicalLink(self.mydoc).canonical_link
        self.assertTrue(mydoc_canonical == self.mydoc.absolute_url(),
            "Steel customized canonical url after remove customization")


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestCanonicalURL))
    return suite



import urllib
from cStringIO import StringIO
from base import *

CUSTOM_METATAGS = [
   {'meta_name': 'metatag1', 'meta_content': 'metatag1value'},
   {'meta_name': 'metatag2', 'meta_content': 'metatag2value'},
   {'meta_name': 'metatag3', 'meta_content': ''}
]

VIEW_METATAGS = [
    'DC.creator', 'DC.format', 'DC.date.modified',
    'DC.date.created', 'DC.type', 'DC.distribution',
    'description', 'keywords', 'robots', 'distribution'
]

FORM = {
    'seo_robots': 'ALL',
    'form.submitted:int': 1,
    'seo_title': 'hello world',
    'seo_title_override:int': 1,
    'seo_robots_override:int': 1,
    'seo_distribution': 'Global',
    'seo_keywords_override:int': 1,
    'seo_keywords:list': 'keyword1',
    'seo_description_override:int': 1,
    'seo_html_comment': 'no comments',
    'seo_html_comment_override:int': 1,
    'seo_distribution_override:int': 1,
    'seo_custommetatags_override:int': 1,
    'seo_description': 'it is description, test keyword1',
}

METATAG = '.*(<meta\s+(?:(?:name="%s"\s*)|(?:content="%s"\s*)){2}/>)'

class TestContextForm(FunctionalTestCase):

    def afterSetUp(self):
        self.sp = self.portal.portal_properties.seo_properties
        self.pu = self.portal.plone_utils
        self.wf = self.portal.portal_workflow

        self.basic_auth = ':'.join((portal_owner,default_password))
        self.loginAsPortalOwner()

        # Preparation for functional testing'
        # create document for test
        my_doc = self.portal.invokeFactory('Document', id='my_doc')
        my_doc = self.portal['my_doc']
        self.abs_path = "/%s" % my_doc.absolute_url(1)
        # prepare seo context form data
        self.sp.manage_changeProperties(
            default_custom_metatags = 'metatag1|global_metatag1value\n' \
                                      'metatag4|global_metatag4value')
        st = ''
        for d in CUSTOM_METATAGS:
            st += '&seo_custommetatags.meta_name:records=%s' % d['meta_name']
            st += '&seo_custommetatags.meta_content:records=%s' % d['meta_content']
        # update seo properties for the test document and publish it
        self.publish(path=self.abs_path+'/@@seo-context-properties',
                     basic=self.basic_auth, request_method='POST',
                     stdin=StringIO(urllib.urlencode(FORM)+st))
        self.wf.doActionFor(my_doc, 'publish')
        # get html view of test document
        self.html = self.publish(self.abs_path, self.basic_auth).getBody()


    def testTitle(self):
        m = re.match('.*<title>\\s*hello world\\s*</title>', self.html, re.S|re.M)
        self.assert_(m, 'Title not set in')

    def testTitleDuplication(self):
        """If we are not overriding page title and current page title equals title of the plone site
        then there should be no concatenation of both titles. Only one should be displayed.
        """
        # setup page with title equal to plone site's title
        my_doc2 = self.portal.invokeFactory('Document', id='my_doc2',
                                            title=self.portal.Title())
        my_doc2 = self.portal['my_doc2']
        self.wf.doActionFor(my_doc2, 'publish')
        html2 = self.publish('/'+my_doc2.absolute_url(1), self.basic_auth).getBody()

        m = re.match('.*<title>\\s*%s\\s*</title>' % self.portal.Title(), html2, re.S|re.M)
        self.assert_(m, 'Title is not set correctly, perhaps it is duplicated with plone site title')

    def testDescription(self):
        m = re.match(METATAG % ("description", FORM['seo_description']), self.html, re.S|re.M)
        self.assert_(m, 'Description not set in')

    def testRobots(self):
        m = re.match(METATAG % ("robots", FORM['seo_robots']), self.html, re.S|re.M)
        self.assert_(m, 'Robots not set in')

    def testDistribution(self):
        m = re.match(METATAG % ("distribution", FORM['seo_distribution']), self.html, re.S|re.M)
        self.assert_(m, 'Distribution not set in')

    def testHTMLComments(self):
        m = re.match('.*<!--\\s*no comments\\s*-->', self.html, re.S|re.M)
        self.assert_(m, 'Comments not set in')

    def testTagsOrder(self):
        def is_match(html, mtorder):
            return re.search('.*'.join(['<meta.*name="%s".*/>' %t \
                                        for t in mtorder]), html, re.S|re.M)

        metatags_order = [t for t in self.sp.getProperty('metatags_order') if t in VIEW_METATAGS]
        self.assert_(is_match(self.html, metatags_order), "Meta tags order not supported.")

        metatags_order.reverse()
        self.assertFalse(is_match(self.html, metatags_order), "Meta tags order not supported.")

        self.sp.manage_changeProperties(metatags_order = metatags_order)
        self.assertFalse(is_match(self.html, metatags_order), "Meta tags order not supported.")

        html = self.publish(self.abs_path, self.basic_auth).getBody()
        self.assert_(is_match(html, metatags_order), "Meta tags order not supported.")


    def testCustomMetaTags(self):
        for tag in CUSTOM_METATAGS:
            m = re.match(METATAG % (tag['meta_name'], tag['meta_content']),
                         self.html, re.S|re.M)
            if tag['meta_content']:
                self.assert_(m, "Custom meta tag %s not applied." % tag['meta_name'])
            else:
                self.assert_(not m, "Meta tag %s has no content, but is present:" \
                                 " in the page." % tag['meta_name'])

        m = re.match(METATAG % ("metatag4", "global_metatag4value"), self.html, re.S|re.M)
        self.assert_(m, "Global custom meta tag %s not applied." % 'metatag4')

    def testDeleteCustomMetaTags(self):
        self.sp.manage_changeProperties(
            default_custom_metatags = 'metatag1|global_metatag1value')
        form_data = {'seo_custommetatags': CUSTOM_METATAGS,
                     'seo_custommetatags_override:int': 0,
                     'form.submitted:int': 1}

        # Clean-up ram cache
        getUtility(IRAMCache).invalidateAll()

        self.publish(path=self.abs_path+'/@@seo-context-properties',
                     basic=self.basic_auth, request_method='POST',
                     stdin=StringIO(urllib.urlencode(form_data)))
        html = self.publish(self.abs_path, self.basic_auth).getBody()

        m = re.match(METATAG % ("metatag4", "global_metatag4value"), html, re.S|re.M)
        self.assert_(not m, "Global custom meta tag %s is prosent in the page." % 'metatag4')

        m = re.match(METATAG % ("metatag1", "global_metatag1value"), html, re.S|re.M)
        self.assert_(m, "Global custom meta tag %s not applied." % 'metatag1')

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestContextForm))
    return suite



from base import *

class TestConfiglet(FunctionalTestCase):

    def afterSetUp(self):
        self.sp = self.portal.portal_properties.site_properties
        self.seo = self.portal.portal_properties.seo_properties
        self.basic_auth = ':'.join((portal_owner,default_password))
        self.loginAsPortalOwner()
        self.save_url = self.portal.id+'/@@seo-controlpanel?form.actions.save=1' \
            '&_authenticator=%s' % self._getauth()

    def test_exposeDCMetaTags_On(self):
        self.publish(self.save_url + '&form.exposeDCMetaTags=on',
                     self.basic_auth)
        self.assert_(self.sp.exposeDCMetaTags)

    def test_exposeDCMetaTags_Off(self):
        self.publish(self.save_url + '&form.exposeDCMetaTags=',
             self.basic_auth)
        self.assertTrue(not self.sp.exposeDCMetaTags)

    def test_defaultCustomMetatags_On(self):
        expect = ('test', 'custom', 'metatags')
        formdata = "\n".join(expect)
        self.publish(self.save_url + '&form.default_custom_metatags=%s'%formdata,
             self.basic_auth)

        dcm = self.seo.getProperty("default_custom_metatags", ())
        self.assertTrue(dcm == expect, '"default_custom_metatags" property ' \
            'contains: "%s", must: "%s"' % (dcm, expect))

    def test_defaultCustomMetatags_Off(self):
        data = ('test', 'custom', 'metatags')
        self.seo._updateProperty("default_custom_metatags", data)
        self.publish(self.save_url + '&form.default_custom_metatags=',
             self.basic_auth)

        dcm = self.seo.getProperty("default_custom_metatags", ())
        self.assertTrue(dcm == (), '"default_custom_metatags" property ' \
            'contains: "%s", must be empty"' % str(dcm))

    def test_metatagsOrder_On(self):
        expect = ('test', 'metatags', 'order')
        formdata = "\n".join(expect)
        self.publish(self.save_url + '&form.metatags_order=%s'%formdata,
             self.basic_auth)

        mo = self.seo.getProperty("metatags_order", ())
        self.assertTrue(mo == expect, '"metatags_order" property ' \
            'contains: "%s", must: "%s"' % (mo, expect))

    def test_metatagsOrder_Off(self):
        data = ('test', 'metatags', 'order')
        self.seo._updateProperty("metatags_order", data)
        self.publish(self.save_url + '&form.metatags_order=',
             self.basic_auth)

        mo = self.seo.getProperty("metatags_order", ())
        self.assertTrue(mo == (), '"metatags_order" property ' \
            'contains: "%s", must be empty"' % str(mo))

    def test_typesSEOEnabled_On(self):
        expect = 'Event'
        self.publish(self.save_url + "&form.types_seo_enabled=%s" % expect, self.basic_auth)
        tse = self.seo.getProperty("content_types_with_seoproperties", ())
        self.assertTrue(tse == (expect,),
            '"content_types_with_seoproperties" property contains: ' \
            '"%s", must: "%s"' % (tse, expect))

    def test_typesSEOEnabled_Off(self):
        self.publish(self.save_url + '&form.types_seo_enabled-empty-marker=1',
             self.basic_auth)

        tse = self.seo.getProperty("content_types_with_seoproperties", ())
        self.assertTrue(tse == (), '"content_types_with_seoproperties" property ' \
            'contains: "%s", must be empty"' % str(tse))

    def test_CustomScriptAdd(self):
        expect = "<script>\n<!-- Test custom script -->\n</script>"

        self.publish(self.save_url + '&form.custom_script=%s' % expect,
             self.basic_auth)

        cs = self.seo.getProperty("custom_script", "")
        self.assertTrue(cs == expect, '"custom_script" property ' \
            'contains: "%s", but "%s" must be"' % (cs, expect))

    def test_CustomScriptDel(self):
        self.publish(self.save_url + '&form.custom_script=',
             self.basic_auth)

        cs = self.seo.getProperty("custom_script", "")
        self.assertTrue(cs == "", '"custom_script" property ' \
            'contains: "%s", must be empty"' % cs)

    def test_fieldsAdd(self):
        expect = ('field1', 'field2')
        formdata = "\n".join(expect)
        self.publish(self.save_url + '&form.fields=%s'%formdata,
             self.basic_auth)

        f = self.seo.getProperty("fields", ())
        self.assertTrue(f == expect, '"fields" property ' \
            'contains: "%s", must: "%s"' % (f, expect))

    def test_fieldsDel(self):
        data = ('field1', 'field2')
        self.seo._updateProperty("fields", data)
        self.publish(self.save_url + '&form.fields=',
             self.basic_auth)

        f = self.seo.getProperty("fields", ())
        self.assertTrue(f == (), '"fields" property ' \
            'contains: "%s", must be empty"' % str(f))

    def test_stopWordsAdd(self):
        expect = ('sw1', 'sw2', 'sw3')
        formdata = "\n".join(expect)
        self.publish(self.save_url + '&form.stop_words=%s'%formdata,
             self.basic_auth)

        f = self.seo.getProperty("stop_words", ())
        self.assertTrue(f == expect, '"stop_words" property ' \
            'contains: "%s", must: "%s"' % (f, expect))

    def test_stopWordsDel(self):
        data = ('sw1', 'sw2', 'sw3')
        self.seo._updateProperty("stop_words", data)
        self.publish(self.save_url + '&form.stop_words=',
             self.basic_auth)

        f = self.seo.getProperty("stop_words", ())
        self.assertTrue(f == (), '"stop_words" property ' \
            'contains: "%s", must be empty"' % str(f))


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestConfiglet))
    return suite



import urllib
from cStringIO import StringIO

from OFS.interfaces import ITraversable

from zope.component import providedBy
from zope.component import getGlobalSiteManager
from zope.component import queryAdapter, getMultiAdapter
from zope.interface import directlyProvides
from zope.viewlet.interfaces import IViewlet, IViewletManager
from zope.publisher.browser import TestRequest

from quintagroup.seoptimizer.browser.interfaces import IPloneSEOLayer
from quintagroup.canonicalpath.interfaces import ICanonicalLink
from quintagroup.canonicalpath.adapters import DefaultCanonicalLinkAdapter
from base import *


class TestBugs(FunctionalTestCase):

    def afterSetUp(self):
        self.basic_auth = ':'.join((portal_owner,default_password))
        self.loginAsPortalOwner()
        # prepare test document
        my_doc = self.portal.invokeFactory('Document', id='my_doc')
        self.my_doc = self.portal['my_doc']
        self.mydoc_path = "/%s" % self.my_doc.absolute_url(1)

    def test_modification_date(self):
        """ Modification date changing on SEO properties edit """
        form_data = {'seo_title': 'New Title',
                     'seo_title_override:int': 1,
                     'form.submitted:int': 1}

        md_before = self.my_doc.modification_date
        self.publish(path=self.mydoc_path+'/@@seo-context-properties',
                     basic=self.basic_auth, request_method='POST',
                     stdin=StringIO(urllib.urlencode(form_data)))
        md_after = self.my_doc.modification_date

        self.assertNotEqual(md_before, md_after)

    def test_bug_20_at_plone_org(self):
        portal = self.portal
        fp = portal['front-page']
        request = portal.REQUEST
        view = portal.restrictedTraverse('@@plone')

        manager = getMultiAdapter((fp, request, view), IViewletManager,
                        name=u'plone.htmlhead')
        viewlet = getMultiAdapter((fp, request, view, manager), IViewlet,
                        name=u'plone.htmlhead.title')
        viewlet.update()
        old_title = viewlet.render()

        # add IPloneSEOLayer
        directlyProvides(request, IPloneSEOLayer)

        viewlet = getMultiAdapter((fp, request, view, manager), IViewlet,
                        name=u'plone.htmlhead.title')
        viewlet.update()
        new_title = viewlet.render()

        self.assertEqual(old_title, new_title)

    def test_bug_22_at_plone_org(self):
        """If ICanonicalLink adapter is not found for the context object
           - page rendering should not break, but only canonical link
           should disappear.
        """
        curl = re.compile('<link\srel\s*=\s*"canonical"\s+' \
                         '[^>]*href\s*=\s*\"([^\"]*)\"[^>]*>', re.S|re.M)
        # When adapter registered for the object - canoncal link present on the page
        self.assertNotEqual( queryAdapter(self.my_doc, ICanonicalLink), None)

        res = self.publish(path=self.mydoc_path, basic=self.basic_auth)
        self.assertNotEqual(curl.search(res.getBody()), None)

        # Now remove adapter from the registry -> this should :
        #     - not break page on rendering;
        #     - canonical link will be absent on the page
        gsm = getGlobalSiteManager()
        gsm.unregisterAdapter(DefaultCanonicalLinkAdapter, [ITraversable,],
                              ICanonicalLink)
        self.assertEqual( queryAdapter(self.my_doc, ICanonicalLink), None)

        res = self.publish(path=self.mydoc_path, basic=self.basic_auth)
        self.assertEqual(curl.search(res.getBody()), None)

        # register adapter back in the global site manager
        gsm.registerAdapter(DefaultCanonicalLinkAdapter, [ITraversable,],
                            ICanonicalLink)


    def test_bug_19_23_at_plone_org(self):
        """overrides.zcml should present in the root of the package"""
        import quintagroup.seoptimizer
        try:
            zcml.load_config('overrides.zcml', quintagroup.seoptimizer)
        except IOError:
            self.fail("overrides.zcml removed from the package root")


class TestBug24AtPloneOrg(FunctionalTestCase):

    def afterSetUp(self):
        # Add test users: member, editor
        member_id = 'test_member'
        editor_id = 'test_editor'
        test_pswd = 'pswd'
        uf = self.portal.acl_users
        uf.userFolderAddUser(member_id, test_pswd,
                        ['Member'], [])
        uf.userFolderAddUser(editor_id, test_pswd,
                        ['Member','Editor'], [])

        self.member_auth = '%s:%s'%(member_id, test_pswd)
        self.editor_auth = '%s:%s'%(editor_id, test_pswd)

        self.portal_url = '/'.join(self.portal.getPhysicalPath())


    def test_not_break(self):
        """Default portal page should not breaks for any user"""
        # Anonymous
        resp = self.publish(path=self.portal_url)
        self.assertEqual(resp.getStatus(), 200)
        # Member
        resp = self.publish(path=self.portal_url, basic=self.member_auth)
        self.assertEqual(resp.getStatus(), 200)
        # Editor: this fails, althought must pass
        resp = self.publish(path=self.portal_url, basic=self.editor_auth)
        self.assertEqual(resp.getStatus(), 200)

    def test_tab_visibility(self):
        """Only Editor can view seo tab"""
        rexp = re.compile('<a\s+[^>]*' \
               'href="[a-zA-Z0-9\:\/_-]*/@@seo-context-properties"[^>]*>'\
               '\s*SEO Properties\s*</a>', re.I|re.S)
        # Anonymous: NO SEO Properties link
        res = self.publish(path=self.portal_url).getBody()
        self.assertEqual(rexp.search(res), None)
        # Member: NO 'SEO Properties' link
        res = self.publish(path=self.portal_url, basic=self.member_auth).getBody()
        self.assertEqual(rexp.search(res), None)
        # Editor: PRESENT 'SEO Properties' link
        res = self.publish(path=self.portal_url, basic=self.editor_auth).getBody()
        self.assertNotEqual(rexp.search(res), None)

    def test_tab_access(self):
        """Only Editor can access 'SEO Properties' tab"""
        test_url = self.portal_url + '/front-page/@@seo-context-properties'
        # Anonymous: can NOT ACCESS
        headers = self.publish(path=test_url).headers
        self.assertEqual( headers.get('bobo-exception-type',""), 'Unauthorized',
            "No 'Unauthorized' exception rised for Anonymous on '@@seo-context-properties' view")
        # Member: can NOT ACCESS
        status = self.publish(path=test_url, basic=self.member_auth).headers
        self.assertEqual( headers.get('bobo-exception-type',""), 'Unauthorized',
            "No 'Unauthorized' exception rised for Member on '@@seo-context-properties' view")
        # Editor: CAN Access
        res = self.publish(path=test_url, basic=self.editor_auth)
        self.assertEqual(res.status, 200)


    def test_tab_edit(self):
        """Editor can change SEO Properties"""
        test_url = self.portal_url + '/front-page/@@seo-context-properties'
        form_data = {'seo_title': 'New Title',
                     'seo_title_override:int': 1,
                     'form.submitted:int': 1}
        res = self.publish(path=test_url, basic=self.editor_auth,
                  request_method='POST', stdin=StringIO(urllib.urlencode(form_data)))
        self.assertNotEqual(res.status, 200)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestBugs))
    suite.addTest(makeSuite(TestBug24AtPloneOrg))
    return suite




import logging
from Products.CMFCore.utils import getToolByName
from Products.GenericSetup.upgrade import _upgrade_registry

from quintagroup.seoptimizer.config import PROJECT_NAME

logger = logging.getLogger('quintagroup.seoptimizer')

# our GenericSetup profile names
INSTALL = 'profile-%s:default' % PROJECT_NAME
UNINSTALL = 'profile-%s:uninstall' % PROJECT_NAME

def install(portal, reinstall=False):
    """ (Re)Install this product.

        This external method is need, because portal_quickinstaller doens't know
        what GenericProfile profile to apply when reinstalling a product.
    """
    setup_tool = getToolByName(portal, 'portal_setup')
    if reinstall:
        step = None
        profile_id = 'quintagroup.seoptimizer:default'
        steps_to_run = [s['id'] for s in setup_tool.listUpgrades(profile_id, show_old=False)]
        for step_id in steps_to_run:
            step = _upgrade_registry.getUpgradeStep(profile_id, step_id)
            step.doStep(setup_tool)
            msg = "Ran upgrade step %s for profile %s" % (step.title, profile_id)
            logger.log(logging.INFO, msg)
        # We update the profile version to the last one we have reached
        # with running an upgrade step.
        if step and step.dest is not None and step.checker is None:
           setup_tool.setLastVersionForProfile(profile_id, step.dest)
        return "Ran all reinstall steps."
    else:
        setup_tool.runAllImportStepsFromProfile(INSTALL)
        return "Ran all install steps."

def uninstall(portal, reinstall=False):
    """ Uninstall this product.

        This external method is need, because portal_quickinstaller doens't know
        what GenericProfile profile to apply when uninstalling a product.
    """
    setup_tool = getToolByName(portal, 'portal_setup')
    if reinstall:
        return "Ran all reinstall steps."
    else:
        setup_tool.runAllImportStepsFromProfile(UNINSTALL)
        return "Ran all uninstall steps."



import logging

from zope.component import getSiteManager

from config import SUPPORT_BLAYER
    
from Products.CMFCore.utils import getToolByName

logger = logging.getLogger('quintagroup.seoptimizer')

def removeActions(site):
    """ Remove actions.
    """
    types_tool = getToolByName(site, 'portal_types')
    for ptype in types_tool.objectValues():
        idxs = [idx_act[0] for idx_act in enumerate(ptype.listActions()) \
                           if idx_act[1].id == 'seo_properties']
        if idxs:
            ptype.deleteActions(idxs)
            logger.log(logging.INFO, "Deleted \"SEO Properties\" action for %s type." % ptype.id)

def removeConfiglet(site):
    """ Remove configlet.
    """
    conf_id = 'quintagroup.seoptimizer'
    controlpanel_tool = getToolByName(site, 'portal_controlpanel')
    if controlpanel_tool:
        controlpanel_tool.unregisterConfiglet(conf_id)
        logger.log(logging.INFO, "Unregistered \"%s\" configlet." % conf_id)

def removeBrowserLayer(site):
    """ Remove browser layer.
    """
    if not SUPPORT_BLAYER:
        return

    from plone.browserlayer.utils import unregister_layer
    from plone.browserlayer.interfaces import ILocalBrowserLayerType

    name="qSEOptimizer"
    site = getSiteManager(site)
    registeredLayers = [r.name for r in site.registeredUtilities()
                        if r.provided == ILocalBrowserLayerType]
    if name in registeredLayers:
        unregister_layer(name, site_manager=site)
        logger.log(logging.INFO, "Unregistered \"%s\" browser layer." % name)

def uninstall(context):
    """ Do customized uninstallation.
    """
    if context.readDataFile('seo_uninstall.txt') is None:
        return
    site = context.getSite()
    removeActions(site)
    removeConfiglet(site)
    removeBrowserLayer(site)



import logging
from zope.component import queryMultiAdapter

from quintagroup.canonicalpath.interfaces  import ICanonicalLink
from quintagroup.canonicalpath.adapters import PROPERTY_LINK

logger = logging.getLogger('quintagroup.seoptimizer')
FIX_PTYPES_DOMAIN = ['Document', 'File', 'News Item']
REMOVE_SEOPROPERTIES = ['additional_keywords', 'settings_use_keywords_sg',
                        'settings_use_keywords_lg', 'filter_keywords_by_content',
                       ]

def changeDomain(plone_tools):
    """ Fix i18n_domain bug for some portal_types,
        which present in earlier versions of the package.
    """
    types_tool = plone_tools.types()
    for ptype in [ptypes for ptypes in types_tool.objectValues()
                         if ptypes.id in FIX_PTYPES_DOMAIN]:
        if ptype.i18n_domain == 'quintagroup.seoptimizer':
            ptype.i18n_domain = 'plone'
            logger.log(logging.INFO, "I18n Domain of the type \'%s\' "
                       "changed to \'plone\'." % ptype.id)

def changeMetatagsOrderList(plone_tools):
    """ Change format metatags order list from "metaname accessor" to "metaname".
    """
    types_tool = plone_tools.types()
    setup_tool = plone_tools.url().getPortalObject().portal_setup
    seoprops_tool = plone_tools.properties().seo_properties
    if seoprops_tool.hasProperty('metatags_order'):
        mto = seoprops_tool.getProperty('metatags_order', [])
        mto_new = [line.split(' ')[0].strip() for line in mto]
        if not list(mto) == mto_new:
            logger.log(logging.INFO, "Changed format metatags order list in "
                       "configlet from \"metaname accessor\" to \"metaname\".")
            seoprops_tool.manage_changeProperties(metatags_order=mto_new)
    else:
        setup_tool.runImportStepFromProfile('profile-quintagroup.seoptimizer:default',
                                            'propertiestool')


def migrationActions(plone_tools):
    """ Migration actions from portal_types action to seoproperties tool
        (for seoaction in portal_actions).
    """
    types_tool = plone_tools.types()
    seoprops_tool = plone_tools.properties().seo_properties
    ctws = list(seoprops_tool.getProperty('content_types_with_seoproperties', []))
    flag = False
    for ptype in types_tool.objectValues():
        idxs = [idx_act[0] for idx_act in enumerate(ptype.listActions())
                           if idx_act[1].id == 'seo_properties']
        if idxs:
            if ptype.id not in ctws:
                ctws.append(ptype.id)
                flag = True
            ptype.deleteActions(idxs)
            logger.log(logging.INFO, "Moved \"SEO Properties\" action from %s "
                       "type in portal actions." % ptype.id)
    if flag:
        seoprops_tool.manage_changeProperties(content_types_with_seoproperties=ctws)

def removeNonUseSeoProperties(plone_tools):
    """ Revome properties used in earlier versions of the package
    """
    seoprops_tool = plone_tools.properties().seo_properties
    remove_properties = []
    for pr in REMOVE_SEOPROPERTIES:
      if seoprops_tool.hasProperty(pr):
          remove_properties.append(pr)
          logger.log(logging.INFO, "Removed %s property in seoproperties tool." % pr)
    if not remove_properties:
        seoprops_tool.manage_delProperties(remove_properties)

def removeSkin(plone_tools):
    """ Remove skin layers.
    """
    layer = 'quintagroup.seoptimizer'
    skins_tool = plone_tools.url().getPortalObject().portal_skins
    for skinName in skins_tool.getSkinSelections():
        skin_paths = skins_tool.getSkinPath(skinName).split(',') 
        paths = [l.strip() for l in skin_paths if not (l == layer or l.startswith(layer+'/'))]
        logger.log(logging.INFO, "Removed layers from %s skin." % skinName)
        skins_tool.addSkinSelection(skinName, ','.join(paths))

def migrateCanonical(plone_tools):
    """ Rename qSEO_canonical property into PROPERTY_LINK
        for all portal objects, which use SEO
    """
    types = plone_tools.types()
    portal = plone_tools.url().getPortalObject()
    allCTTypes = types.listContentTypes()
    obj_metatypes =  [m.content_meta_type for m in types.objectValues() \
                      if m.getId() in allCTTypes]
    portal.ZopeFindAndApply(
                            portal,
                            obj_metatypes=','.join(obj_metatypes),
                            apply_func=renameProperty
                            )

def renameProperty(obj, path):
    """ Rename qSEO_canonical property into PROPERTY_LINK
        for obj, which use SEO
    """
    if obj.hasProperty('qSEO_canonical'):
        value = obj.getProperty('qSEO_canonical')

        level, msg = logging.INFO, "For %(url)s object 'qSEO_canonical' "\
                     "property renamed to '%(name)s'."
        try:
            ICanonicalLink(obj).canonical_link = value
        except Exception, e:
            level, msg = logging.ERROR, "%s on renaming 'qSEO_canonical' " \
                         "property for %%(url)s object" % str(e)

        logger.log(level, msg % {'url':obj.absolute_url(), 'name':PROPERTY_LINK} )
        obj.manage_delProperties(['qSEO_canonical'])

def upgrade_2_to_3(setuptool):
    """ Upgrade quintagroup.seoptimizer from version 2.x.x to 3.0.0.
    """
    plone_tools = queryMultiAdapter((setuptool, setuptool.REQUEST), name="plone_tools")
    setuptool.runAllImportStepsFromProfile('profile-quintagroup.seoptimizer:upgrade_2_to_3')
    migrationActions(plone_tools)
    changeDomain(plone_tools)
    changeMetatagsOrderList(plone_tools)
    removeNonUseSeoProperties(plone_tools)
    removeSkin(plone_tools)
    migrateCanonical(plone_tools)



#
# Test product's installation
#
import string
from zope.interface import alsoProvides
from zope.component import queryMultiAdapter
from zope.viewlet.interfaces import IViewletManager

from quintagroup.canonicalpath.adapters import PROPERTY_LINK
from quintagroup.seoptimizer.browser.interfaces import IPloneSEOLayer
from base import *

PROPERTY_SHEET = 'seo_properties'
STOP_WORDS = ['a', 'an', 'amp', 'and', 'are', 'arial', 'as', 'at', 'be', 'but',
    'by', 'can', 'com', 'do', 'font', 'for', 'from', 'gif', 'had', 'has',
    'have', 'he', 'helvetica', 'her', 'his', 'how', 'href', 'i', 'if', 'in',
    'is', 'it', 'javascript', 'jpg', 'made', 'net', 'of', 'on', 'or', 'org',
    'our', 'sans', 'see', 'serif', 'she', 'that', 'the', 'this', 'to', 'us',
    'we', 'with', 'you', 'your']

PROPS = {'stop_words': STOP_WORDS,
         'fields': ['seo_title', 'seo_description', 'seo_keywords']}

DEFAULT_METATAGS_ORDER = [
    'DC.contributors', 'DC.creator', 'DC.date.created',
    'DC.date.modified','DC.description', 'DC.distribution',
    'DC.format', 'DC.language', 'DC.publisher', 'DC.rights',
    'DC.subject', 'DC.type', 'description', 'distribution',
    'keywords', 'robots']
DEFAULT_METATAGS_ORDER.sort()

SEO_CONTENT = ['File', 'Document', 'News Item']
CONTENTTYPES_WITH_SEOACTION = ['File', 'Document', 'News Item', 'Folder', 'Event']
CONTENTTYPES_WITH_SEOACTION.sort()


class TestBeforeInstallation(FunctionalTestCaseNotInstalled):

    def afterSetUp(self):
        self.basic_auth = 'mgr:mgrpw'
        self.portal_path = '/%s' % self.portal.absolute_url(1)

    def testAccessPortalRootAnonymous(self):
        response = self.publish(self.portal_path)
        self.assertEqual(response.getStatus(), 200)

    def testAccessPortalRootAuthenticated(self):
        response = self.publish(self.portal_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)


class TestInstallation(TestCase):

    def afterSetUp(self):
        self.properties = getToolByName(self.portal, 'portal_properties')

    def testAddingPropertySheet(self):
        """ Test adding property sheet to portal_properties tool """
        self.failUnless(hasattr(self.properties.aq_base, PROPERTY_SHEET))

    def testAddingPropertyFields(self):
        """ Test adding property field to portal_properties.maps_properties sheet """
        map_sheet = self.properties[PROPERTY_SHEET]
        for key, value in PROPS.items():
            self.failUnless(map_sheet.hasProperty(key) and list(map_sheet.getProperty(key)) == value)

    def test_configlet_install(self):
        configTool = getToolByName(self.portal, 'portal_controlpanel', None)
        self.assert_(PROJECT_NAME in [a.getId() for a in configTool.listActions()], 'Configlet not found')

    def test_viewlets_install(self):
        VIEWLETS = ['plone.htmlhead.title',
                    'plone.resourceregistries',
                    'quintagroup.seoptimizer.seotags',
                    'quintagroup.seoptimizer.customscript']
        request = self.app.REQUEST
        # mark request with our browser layer
        alsoProvides(request, IPloneSEOLayer)
        view = queryMultiAdapter((self.portal, request), name="plone")
        manager = queryMultiAdapter( (self.portal['front-page'], request, view),
                                     IViewletManager, name='plone.htmlhead')
        for p in VIEWLETS:
            self.assert_(manager.get(p) is not None, "Not registered '%s' viewlet" % p)

    def test_browser_layer(self):
        if not SUPPORT_BLAYER:
            return

        from plone.browserlayer import utils
        self.assert_(IPloneSEOLayer in utils.registered_layers(),
                     "Not registered 'IPloneSEOLayer' browser layer")

    def test_action_install(self):
        atool=getToolByName(self.portal, 'portal_actions')
        action_ids = [a.id for a in atool.listActions()]
        self.assert_("SEOProperties" in action_ids,
                     "Not added 'SEOProperties' action")

class TestUninstallation(TestCase):

    def afterSetUp(self):
        self.qi = self.portal.portal_quickinstaller
        self.qi.uninstallProducts([PROJECT_NAME])

    def test_propertysheet_uninstall(self):
        properties = getToolByName(self.portal, 'portal_properties')
        self.assert_(hasattr(properties.aq_base, PROPERTY_SHEET),
            "'%s' property sheet not uninstalled" % PROPERTY_SHEET)

    def test_configlet_uninstall(self):
        self.assertNotEqual(self.qi.isProductInstalled(PROJECT_NAME), True,
            'qSEOptimizer is already installed')

        configTool = getToolByName(self.portal, 'portal_controlpanel', None)
        self.assertEqual(PROJECT_NAME in [a.getId() for a in configTool.listActions()], False,
            'Configlet found after uninstallation')

    def test_viewlets_uninstall(self):
        VIEWLETS = ['quintagroup.seoptimizer.seotags',
                    'quintagroup.seoptimizer.customscript']
        request = self.app.REQUEST
        view = queryMultiAdapter((self.portal, request), name="plone")
        manager = queryMultiAdapter( (self.portal['front-page'], request, view),
                                     IViewletManager, name='plone.htmlhead')
        for p in VIEWLETS:
            self.assertEqual(manager.get(p) is None, True,
                "'%s' viewlet present after uninstallation" % p)

    def test_browserlayer_uninstall(self):
        if not SUPPORT_BLAYER:
            return

        from plone.browserlayer import utils
        self.assertEqual(IPloneSEOLayer in utils.registered_layers(), False,
            "Still registered 'IPloneSEOLayer' browser layer")

    def test_action_uninstall(self):
        atool=getToolByName(self.portal, 'portal_actions')
        action_ids = [a.id for a in atool.listActions()]
        self.assertEqual("SEOProperties" in action_ids, False,
            "'SEOProperties' action not removed from portal_actions " \
            "on uninstallation")

class TestReinstallation(TestCase):

    def afterSetUp(self):
        self.qi = self.portal.portal_quickinstaller
        self.types_tool = getToolByName(self.portal, 'portal_types')
        self.setup_tool = getToolByName(self.portal, 'portal_setup')
        self.pprops_tool = getToolByName(self.portal, 'portal_properties')
        self.seoprops_tool = getToolByName(self.pprops_tool, 'seo_properties', None)
        # Set earlier version profile (2.0.0) for using upgrade steps 
        self.setup_tool.setLastVersionForProfile('%s:default' % PROJECT_NAME, '2.0.0')

    def testChangeDomain(self):
        # Test changed of content type's domain from 'quintagroup.seoptimizer' to 'plone'
        for type in SEO_CONTENT:
            self.types_tool.getTypeInfo(type).i18n_domain = 'quintagroup.seoptimizer'
        self.qi.reinstallProducts([PROJECT_NAME])
        for type in SEO_CONTENT:
            self.assertEqual(self.types_tool.getTypeInfo(type).i18n_domain, 'plone',
                "Not changed of %s content type's domain to 'plone'" % type)

    def testCutItemsMetatagsOrderList(self):
        # Test changed format metatags order list from "metaname accessor" to "metaname"
        value, expect_mto = ['name1 accessor1', 'name2 accessor2'], ['name1','name2']
        self.seoprops_tool.manage_changeProperties(metatags_order=value)
        self.qi.reinstallProducts([PROJECT_NAME])
        mto = list(self.seoprops_tool.getProperty('metatags_order'))
        mto.sort()
        self.assertEqual(mto, expect_mto,
                    "Not changed format metatags order list from \"metaname accessor\" to"\
                    " \"metaname\". %s != %s" %(mto, expect_mto))

    def testAddMetatagsOrderList(self):
        # Test added metatags order list if it was not there before
        self.seoprops_tool.manage_delProperties(['metatags_order'])
        self.qi.reinstallProducts([PROJECT_NAME])
        mto = list(self.seoprops_tool.getProperty('metatags_order'))
        mto.sort()
        self.assertEqual(mto, DEFAULT_METATAGS_ORDER,
                    "Not added metatags order list with default values."\
                    "%s != %s" %(mto, DEFAULT_METATAGS_ORDER))

    def testMigrationActions(self):
        # Test migrated actions from portal_types action to seoproperties tool
        self.seoprops_tool.content_types_with_seoproperties = ()

        # Add seoaction to content type for testing
        for type in CONTENTTYPES_WITH_SEOACTION:
            self.types_tool.getTypeInfo(type).addAction(id='seo_properties',
                                                   name='SEO Properties',
                                                   action=None,
                                                   condition=None,
                                                   permission=(u'Modify portal content',),
                                                   category='object',
                                                  )
            # Check presence seoaction in content type
            seoaction = [act.id for act in self.types_tool.getTypeInfo(type).listActions()
                                          if act.id == 'seo_properties']
            self.assertEqual(bool(seoaction), True,
                    "Not added seoaction to content type %s for testing" % type)

        self.qi.reinstallProducts([PROJECT_NAME])

        # Check presence seoaction in content type
        for type in CONTENTTYPES_WITH_SEOACTION:
            seoaction = [act.id for act in self.types_tool.getTypeInfo(type).listActions()
                                          if act.id == 'seo_properties']
            self.assertEqual(bool(seoaction), False,
                "Not removed seoaction in content type %s" % type)

        # Check added content type names in seo properties tool if content types have seoaction
        ctws = list(self.seoprops_tool.content_types_with_seoproperties)
        ctws.sort()
        self.assertEqual(ctws, CONTENTTYPES_WITH_SEOACTION,
            "Not added content type names in seo properties tool if content types have seoaction."\
            " %s != %s" %(ctws, CONTENTTYPES_WITH_SEOACTION))

    def testRemoveSkin(self):
        # Test remove layers
        layer = 'quintagroup.seoptimizer'
        skinstool = getToolByName(self.portal, 'portal_skins')
        for skin in skinstool.getSkinSelections():
            paths  = ','.join((skinstool.getSkinPath(skin), layer))
            skinstool.addSkinSelection(skin, paths)
        self.qi.reinstallProducts([PROJECT_NAME])
        for skin in skinstool.getSkinSelections():
            path = skinstool.getSkinPath(skin)
            path = map(string.strip, string.split(path, ','))
            self.assertEqual(layer in path, False,
                '%s layer found in %s after uninstallation' %(layer, skin))

    def testMigrateCanonical(self):
        """ Test Migrate qSEO_canonical property into PROPERTY_LINK
            for all portal objects, which use SEO
        """
        doc = self.portal.get('front-page')
        doc.manage_addProperty('qSEO_canonical', 'val', 'string')
        value = doc.getProperty('qSEO_canonical')
        assert doc.getProperty('qSEO_canonical') == 'val' 

        self.qi.reinstallProducts([PROJECT_NAME])
        value = doc.getProperty(PROPERTY_LINK)
        has_prop = bool(doc.hasProperty('qSEO_canonical'))
        self.assertEqual(has_prop, False, "Property 'qSEO_canonical' is not deleted.")
        self.assertEqual(value == 'val', True,
                "Property not migrated from 'qSEO_canonical' to '%s'." % PROPERTY_LINK)

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestBeforeInstallation))
    suite.addTest(makeSuite(TestInstallation))
    suite.addTest(makeSuite(TestUninstallation))
    suite.addTest(makeSuite(TestReinstallation))
    return suite



import re
from zope.interface import Interface
from zope.interface import implements
from zope.component import adapts
from zope.schema import Bool, Choice, Tuple, List
from zope.schema import SourceText

from zope.app.component.hooks import getSite
from zope.app.form.browser import TextAreaWidget

from plone.fieldsets.fieldsets import FormFieldsets
from plone.app.controlpanel.form import ControlPanelForm
from plone.app.controlpanel.widgets import MultiCheckBoxThreeColumnWidget

from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import safe_unicode
from Products.CMFDefault.formlib.schema import ProxyFieldProperty
from Products.CMFDefault.formlib.schema import SchemaAdapterBase
from Products.CMFPlone.interfaces import IPloneSiteRoot

from quintagroup.seoptimizer import SeoptimizerMessageFactory as _


# Configlet schemas
class ISEOConfigletBaseSchema(Interface):

    exposeDCMetaTags = Bool(
        title=_("label_exposeDCMetaTags",
                default='Expose <abbr title="Dublin Core">DC</abbr> meta tags'),
        description=_("description_seo_dc_metatags",
                default='Controls if <abbr title="Dublin Core">DC</abbr> '
                    'metatags are exposed to page header. They include '
                    'DC.description, DC.type, DC.format, DC.creator and '
                    'others.'),
        default=True,
        required=False)

    metatags_order = List(
        title=_("label_metatags_order",
                default='Meta tags order in the page.'),
        description=_("help_metatags_order",
                default='Fill in meta tags (one per line) in the order in which'
                    ' they will appear on site source pages. Example: '
                    '"metaname accessor".'),
        required=False)

    types_seo_enabled = Tuple(
        title=_("label_content_type_title", default='Content Types'),
        description=_("description_seo_content_types",
            default='Select content types that will have SEO properties '
                'enabled.'),
        required=False,
        missing_value=tuple(),
        value_type=Choice(
            vocabulary="plone.app.vocabularies.ReallyUserFriendlyTypes"))

    default_custom_metatags = List(
        title=_("label_default_custom_metatags", default='Default custom metatags.'),
        description=_("help_default_custom_metatags",
                default='Fill in custom metatag names (one per line) which will '
                    'appear on qseo_properties edit tab. Example: '
                    '"metaname|metacontent" or "metaname".'),
        required=False)



class ISEOConfigletAdvancedSchema(Interface):
    custom_script = SourceText(
        title=_("label_custom_script", default=u'Header JavaScript'),
        description=_("help_custom_script",
                default=u"This JavaScript code will be included in "
                         "the rendered HTML as entered in the page header."),
        default=u'',
        required=False)

    fields = List(
        title=_("label_fields", default='Fields for keywords statistic calculation.'),
        description=_("help_fields",
                default='Fill in filds (one per line) which statistics of keywords usage '
                    'should be calculated for.'),
        required=False)

    stop_words = List(
        title=_("label_stop_words", default='Stop words.'),
        description=_("help_stop_words",
                default='Fill in stop words (one per line) which will '
                    'be excluded from kewords statistics calculation.'),
        required=False)



class ISEOConfigletSchema(ISEOConfigletBaseSchema,
                          ISEOConfigletAdvancedSchema):
    """Combined schema for the adapter lookup.
    """


class SEOConfigletAdapter(SchemaAdapterBase):

    adapts(IPloneSiteRoot)
    implements(ISEOConfigletSchema)

    def __init__(self, context):
        super(SEOConfigletAdapter, self).__init__(context)
        self.portal = getSite()
        pprop = getToolByName(self.portal, 'portal_properties')
        self.context = pprop.seo_properties
        self.siteprops = pprop.site_properties
        self.ttool = getToolByName(context, 'portal_types')
        self.encoding = pprop.site_properties.default_charset


    def getExposeDC(self):
        return self.siteprops.getProperty('exposeDCMetaTags')

    def setExposeDC(self, value):
        return self.siteprops._updateProperty('exposeDCMetaTags', bool(value))

    def getTypesSEOEnabled(self):
        ct_with_seo = self.context.content_types_with_seoproperties
        return [t for t in self.ttool.listContentTypes() if t in ct_with_seo]

    def setTypesSEOEnabled(self, value):
        value = [t for t in self.ttool.listContentTypes() if t in value]
        self.context._updateProperty('content_types_with_seoproperties', value)

    def getCustomScript(self):
        description = getattr(self.context, 'custom_script', u'')
        return safe_unicode(description)

    def setCustomScript(self, value):
        if value is not None:
            self.context.custom_script = value.encode(self.encoding)
        else:
            self.context.custom_script = ''

    exposeDCMetaTags = property(getExposeDC, setExposeDC)
    default_custom_metatags = ProxyFieldProperty(ISEOConfigletSchema['default_custom_metatags'])
    metatags_order = ProxyFieldProperty(ISEOConfigletSchema['metatags_order'])
    types_seo_enabled = property(getTypesSEOEnabled, setTypesSEOEnabled)
    custom_script = property(getCustomScript, setCustomScript)
    fields = ProxyFieldProperty(ISEOConfigletSchema['fields'])
    stop_words = ProxyFieldProperty(ISEOConfigletSchema['stop_words'])


class Text2ListWidget(TextAreaWidget):
    height = 5
    splitter = re.compile(u'\\r?\\n', re.S|re.U)

    def _toFieldValue(self, input):
        if input == self._missing:
            return self.context._type()
        else:
            return self.context._type(filter(None, self.splitter.split(input)))

    def _toFormValue(self, value):
        if value == self.context.missing_value or value == self.context._type():
            return self._missing
        else:
            return u'\r\n'.join(list(value))


# Fieldset configurations
baseset = FormFieldsets(ISEOConfigletBaseSchema)
baseset.id = 'seobase'
baseset.label = _(u'label_seobase', default=u'Base')

advancedset = FormFieldsets(ISEOConfigletAdvancedSchema)
advancedset.id = 'seoadvanced'
advancedset.label = _(u'label_seoadvanced', default=u'Advanced')

class SEOConfiglet(ControlPanelForm):

    form_fields = FormFieldsets(baseset, advancedset)

    form_fields['default_custom_metatags'].custom_widget = Text2ListWidget
    form_fields['metatags_order'].custom_widget = Text2ListWidget
    form_fields['types_seo_enabled'].custom_widget = MultiCheckBoxThreeColumnWidget
    form_fields['types_seo_enabled'].custom_widget.cssClass='label'
    form_fields['fields'].custom_widget = Text2ListWidget
    form_fields['stop_words'].custom_widget = Text2ListWidget

    label = _("Search Engine Optimizer configuration")
    description = _("seo_configlet_description", default="You can select what "
                    "content types are qSEOptimizer-enabled, and control if "
                    "Dublin Core metatags are exposed in the header of content "
                    "pages.")
    form_name = _("")



from base import *

GENERATOR = re.compile('.*(<meta\s+(?:(?:name="generator"\s*)|' \
                       '(?:content=".*?"\s*)){2}/>)', re.S|re.M)
DESCRIPTION = re.compile('.*(<meta\s+(?:(?:name="description"\s*)|' \
                         '(?:content=".*?"\s*)){2}/>)', re.S|re.M)

class InstallMixin:

    def prepare(self):
        # Preparation for functional testing
        self.loginAsPortalOwner()
        self.my_doc = self.portal.invokeFactory('Document', id='my_doc')
        self.my_doc = self.portal['my_doc']
        self.my_doc.update(description="Document description")
        self.portal.portal_workflow.doActionFor(self.my_doc, 'publish')
        self.logout()
        # Get document without customized canonical url
        self.abs_path = "/%s" % self.my_doc.absolute_url(1)
        self.html = self.publish(self.abs_path).getBody()


class TestTagsDuplicationInstalled(InstallMixin, FunctionalTestCase):

    def afterSetUp(self):
        self.prepare()

    def test_GeneratorMetaSEOInstalled(self):
        lengen = len(GENERATOR.findall(self.html))
        self.assert_(lengen==1, "There is %d generator meta tag(s) " \
           "when seoptimizer installed" % lengen)
 
    def test_DescriptionMetaSEOInstalled(self):
        lendesc = len(DESCRIPTION.findall(self.html))
        self.assert_(lendesc==1, "There is %d DESCRIPTION meta tag(s) " \
           "when seoptimizer installed" % lendesc)


class TestTagsDuplicationNotInstalled(InstallMixin,
                                      FunctionalTestCaseNotInstalled):

    def afterSetUp(self):
        self.prepare()

    def test_GeneratorMetaSEOUninstalled(self):
        lengen = len(GENERATOR.findall(self.html))
        self.assert_(lengen<=1, "There is %d generator meta tag(s) " \
            "when seoptimizer uninstalled" % lengen)

    def test_DescriptionMetaSEOUninstalled(self):
        lendesc = len(DESCRIPTION.findall(self.html))
        self.assert_(lendesc==1, "There is %d DESCRIPTION meta tag(s) " \
           "when seoptimizer uninstalled" % lendesc)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestTagsDuplicationInstalled))
    suite.addTest(makeSuite(TestTagsDuplicationNotInstalled))
    return suite



import string
from DateTime import DateTime
from base import *

METATAG = '.*(<meta\s+(?:(?:name="%s"\s*)|(?:content="(?P<tagcontent>.*?)"\s*)){2}/>)'

class TestExposeDCMetaTags(FunctionalTestCase):

    def afterSetUp(self):
        self.sp = self.portal.portal_properties.site_properties
        self.basic_auth = ':'.join((portal_owner,default_password))
        self.loginAsPortalOwner()
        # Preparation for functional testing
        self.my_doc = self.portal.invokeFactory('Document', id='my_doc')
        self.my_doc = self.portal['my_doc']

    def test_propertyOff(self):
        self.sp.manage_changeProperties(exposeDCMetaTags = False)
        self.html = str(self.publish(self.portal.id+'/my_doc', self.basic_auth))
        m1 = re.match(METATAG % "DC.format", self.html, re.S|re.M)
        m2 = re.match(METATAG % "DC.distribution", self.html, re.S|re.M)
        self.assert_(not (m1 or m2), 'DC meta tags avaliable when exposeDCMetaTags=False')

    def test_propertyOn(self):
        self.sp.manage_changeProperties(exposeDCMetaTags = True)
        self.html = str(self.publish(self.portal.id+'/my_doc', self.basic_auth))
        m1 = re.match(METATAG % "DC.format", self.html, re.S|re.M)
        m2 = re.match(METATAG % "DC.type", self.html, re.S|re.M)
        self.assert_(m1 and m2, 'DC meta tags not avaliable when createManager=True')

    def test_descriptionInPropertyOff(self):
        self.sp.manage_changeProperties(exposeDCMetaTags = False)
        self.my_doc.setDescription("My document description")
        self.html = str(self.publish(self.portal.id+'/my_doc', self.basic_auth))
        m = re.match(METATAG % "description", self.html, re.S|re.M)
        self.assert_(m, 'No "description" meta tag when expose DC meta tags is Off')

    def test_descriptionInPropertyOn(self):
        self.sp.manage_changeProperties(exposeDCMetaTags = True)
        self.my_doc.setDescription("My document description")
        self.html = str(self.publish(self.portal.id+'/my_doc', self.basic_auth))
        m = re.match(METATAG % "description", self.html, re.S|re.M)
        self.assert_(m, 'No "description" meta tag when expose DC meta tags is On')

    def test_dateValidRange(self):
        self.sp.manage_changeProperties(exposeDCMetaTags = True)
        EFFDSTR, EXPDSTR =  "2009/12/23", "2010/03/10"
        self.my_doc.setExpirationDate(EXPDSTR)
        self.my_doc.setEffectiveDate(EFFDSTR)
        self.html = str(self.publish(self.portal.id+'/my_doc', self.basic_auth))
        m = re.match(METATAG % "DC.date.valid_range", self.html, re.S|re.M)
        content = m and m.group("tagcontent")
        fact = content and map(DateTime, content.split("-"))
        expect = map(DateTime, [EFFDSTR, EXPDSTR])
        self.assert_( fact == expect, '"DC.date.valid_range" meta tags content="%s", ' \
                      'but "%s" must be' % ( fact, expect ))

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestExposeDCMetaTags))
    return suite



from time import time
from DateTime import DateTime
from Acquisition import aq_inner
from zope.component import queryAdapter
from zope.component import queryMultiAdapter
from zope.schema.interfaces import InvalidValue

from plone.memoize import view, ram

from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFPlone.utils import getSiteEncoding

from quintagroup.canonicalpath.interfaces import ICanonicalLink
from quintagroup.canonicalpath.adapters import PROPERTY_LINK as CANONICAL_PROPERTY

from quintagroup.seoptimizer.browser.seo_configlet import ISEOConfigletSchema
from quintagroup.seoptimizer import SeoptimizerMessageFactory as _

SEPERATOR = '|'
SEO_PREFIX = 'seo_'
PROP_PREFIX = 'qSEO_'
SUFFIX = '_override'
PROP_CUSTOM_PREFIX = 'qSEO_custom_'

# Ram cache function, which depends on plone instance and time
def plone_instance_time(method, self, *args, **kwargs):
    return (self.pps.portal(), time() // (60 * 60))

class SEOContext( BrowserView ):
    """ This class contains methods that allows to edit html header meta tags.
    """

    def __init__(self, *args, **kwargs):
        super(SEOContext, self).__init__(*args, **kwargs)
        self.pps = queryMultiAdapter((self.context, self.request), name="plone_portal_state")
        self.pcs = queryMultiAdapter((self.context, self.request), name="plone_context_state")
        self.gseo = queryAdapter(self.pps.portal(), ISEOConfigletSchema)
        self._seotags = self._getSEOTags()

    def __getitem__(self, key):
        return self._seotags.get(key, '')

    @view.memoize
    def _getSEOTags(self):
        seotags = {
            "seo_title": self.getSEOProperty( 'qSEO_title', default=self.pcs.object_title() ),
            "seo_robots": self.getSEOProperty( 'qSEO_robots', default='ALL'),
            "seo_description": self.getSEOProperty( 'qSEO_description', accessor='Description' ),
            "seo_distribution": self.getSEOProperty( 'qSEO_distribution', default="Global"),
            "seo_customMetaTags": self.seo_customMetaTags(),
            # "seo_localCustomMetaTags": self.seo_localCustomMetaTags(),
            # "seo_globalCustomMetaTags": self.seo_globalCustomMetaTags(),
            "seo_html_comment": self.getSEOProperty( 'qSEO_html_comment', default='' ),
            "meta_keywords": self.getSEOProperty('qSEO_keywords', 'Subject', ()),
            "seo_keywords": self.getSEOProperty('qSEO_keywords', default=()),
            "seo_canonical": self.getCanonical(),
            # Add test properties
            "has_seo_title": self.context.hasProperty('qSEO_title'),
            "has_seo_robots": self.context.hasProperty('qSEO_robots'),
            "has_seo_description": self.context.hasProperty( 'qSEO_description'),
            "has_seo_distribution": self.context.hasProperty( 'qSEO_distribution'),
            "has_html_comment": self.context.hasProperty('qSEO_html_comment'),
            "has_seo_keywords": self.context.hasProperty('qSEO_keywords'),
            "has_seo_canonical": self.context.hasProperty(CANONICAL_PROPERTY),
            }
        #seotags["seo_nonEmptylocalMetaTags"] = bool(seotags["seo_localCustomMetaTags"])
        return seotags

    def getSEOProperty( self, property_name, accessor='', default=None ):
        """ Get value from seo property by property name.
        """
        context = aq_inner(self.context)

        if context.hasProperty(property_name):
            return context.getProperty(property_name, default)

        if accessor:
            method = getattr(context, accessor, default)
            if not callable(method):
                return default

            # Catch AttributeErrors raised by some AT applications
            try:
                value = method()
            except AttributeError:
                value = default

            return value
        return default

    def seo_customMetaTags( self ):
        """Returned seo custom metatags from default_custom_metatags property in seo_properties
           (global seo custom metatags) with update from seo custom metatags properties
           in context (local seo custom metatags).
        """
        glob, loc = self.seo_globalCustomMetaTags(), self.seo_localCustomMetaTags()
        gnames = set(map(lambda x: x['meta_name'], glob))
        lnames = set(map(lambda x: x['meta_name'], loc))
        # Get untouch global, override global in custom and new custom meta tags
        untouchglob = [t for t in glob if t['meta_name'] in list(gnames - lnames)]
        return untouchglob + loc

    def seo_globalWithoutLocalCustomMetaTags( self ):
        """Returned seo custom metatags from default_custom_metatags property in seo_properties
           (global seo custom metatags) without seo custom metatags from properties
           in context (local seo custom metatags).
        """
        glob, loc = self.seo_globalCustomMetaTags(), self.seo_localCustomMetaTags()
        gnames = set(map(lambda x: x['meta_name'], glob))
        lnames = set(map(lambda x: x['meta_name'], loc))
        return [t for t in glob if t['meta_name'] in list(gnames - lnames)]

    def seo_localCustomMetaTags( self ):
        """ Returned seo custom metatags from properties in context (local seo custom metatags).
        """
        result = []
        property_prefix = 'qSEO_custom_'
        context = aq_inner(self.context)
        for property, value in context.propertyItems():
            if property.startswith(property_prefix) and property[len(property_prefix):]:
                result.append({'meta_name'    : property[len(property_prefix):],
                               'meta_content' : value})
        return result

    @ram.cache(plone_instance_time)
    def seo_globalCustomMetaTags( self ):
        """ Returned seo custom metatags from default_custom_metatags property in seo_properties.
        """
        result = []
        if self.gseo:
            for tag in self.gseo.default_custom_metatags:
                name_value = tag.split(SEPERATOR)
                if name_value[0]:
                    result.append({'meta_name'    : name_value[0],
                                   'meta_content' : len(name_value) == 2 and name_value[1] or ''})
        return result

    def getCanonical(self):
        canonical = queryAdapter(self.context, ICanonicalLink)
        return canonical and canonical.canonical_link or ""


class SEOContextPropertiesView( BrowserView ):
    """ This class contains methods that allows to manage seo properties.
    """
    template = ViewPageTemplateFile('templates/seo_context_properties.pt')

    def __init__(self, *args, **kwargs):
        super(SEOContextPropertiesView, self).__init__(*args, **kwargs)
        self.pps = queryMultiAdapter((self.context, self.request),
                                     name="plone_portal_state")
        self.gseo = queryAdapter(self.pps.portal(), ISEOConfigletSchema)


    def test( self, condition, first, second ):
        """
        """
        return condition and first or second

    def validateSEOProperty(self, property, value):
        """ Validate a seo property.
        """
        return ''

    def setProperty(self, property, value, type='string'):
        """ Add a new property.

            Sets a new property with the given id, value and type or changes it.
        """
        context = aq_inner(self.context)
        state = self.validateSEOProperty(property, value)
        if not state:
            if context.hasProperty(property):
                context.manage_changeProperties({property: value})
            else:
                context.manage_addProperty(property, value, type)
        return state

    def manageSEOProps(self, **kw):
        """ Manage seo properties.
        """
        context = aq_inner(self.context)
        state = ''
        delete_list, seo_overrides_keys, seo_keys = [], [], []
        seo_items = dict([(k[len(SEO_PREFIX):],v) for k,v in kw.items() if k.startswith(SEO_PREFIX)])
        for key in seo_items.keys():
            if key.endswith(SUFFIX):
                seo_overrides_keys.append(key[:-len(SUFFIX)])
            else:
                seo_keys.append(key)
        for seo_key in seo_keys:
            if seo_key == 'custommetatags':
                self.manageSEOCustomMetaTagsProperties(**kw)
            else:
                if seo_key in seo_overrides_keys and seo_items.get(seo_key+SUFFIX):
                    seo_value = seo_items[seo_key]
                    if seo_key == 'canonical':
                        try:
                            ICanonicalLink(self.context).canonical_link = seo_value
                        except InvalidValue, e:
                            state = "'%s' - wrong canonical url" % str(e)
                    else:
                        t_value = 'string'
                        if type(seo_value)==type([]) or type(seo_value)==type(()): t_value = 'lines'
                        state = self.setProperty(PROP_PREFIX+seo_key, seo_value, type=t_value)
                    if state:
                        return state
                elif seo_key == 'canonical':
                    del ICanonicalLink(self.context).canonical_link
                elif context.hasProperty(PROP_PREFIX+seo_key):
                    delete_list.append(PROP_PREFIX+seo_key)
        if delete_list:
            context.manage_delProperties(delete_list)
        return state

    def setSEOCustomMetaTags(self, custommetatags):
        """ Set seo custom metatags properties.
        """
        context = aq_inner(self.context)
        for tag in custommetatags:
            self.setProperty('%s%s' % (PROP_CUSTOM_PREFIX, tag['meta_name']), tag['meta_content'])

    def delAllSEOCustomMetaTagsProperties(self):
        """ Delete all seo custom metatags properties.
        """
        context = aq_inner(self.context)
        delete_list = []
        for property, value in context.propertyItems():
            if property.startswith(PROP_CUSTOM_PREFIX)  and not property == PROP_CUSTOM_PREFIX:
                delete_list.append(property)
        if delete_list:
            context.manage_delProperties(delete_list)

    def updateSEOCustomMetaTagsProperties(self, custommetatags):
        """ Update seo custom metatags properties.
        """
        globalCustomMetaTags = []
        if self.gseo:
            custom_meta_tags = self.gseo.default_custom_metatags
            for tag in custom_meta_tags:
                name_value = tag.split(SEPERATOR)
                if name_value[0]:
                    globalCustomMetaTags.append(
                        {'meta_name' : name_value[0],
                         'meta_content' : len(name_value) == 1 and '' or name_value[1]})
        for tag in custommetatags:
            meta_name, meta_content = tag['meta_name'], tag['meta_content']
            if meta_name:
                if not [gmt for gmt in globalCustomMetaTags \
                        if (gmt['meta_name']==meta_name and gmt['meta_content']==meta_content)]:
                    self.setProperty('%s%s' % (PROP_CUSTOM_PREFIX, meta_name), meta_content)

    def manageSEOCustomMetaTagsProperties(self, **kw):
        """ Update seo custom metatags properties, if enabled checkbox override or delete properties.

            Change object properties by passing either a mapping object
            of name:value pairs {'foo':6} or passing name=value parameters.
        """
        context = aq_inner(self.context)
        self.delAllSEOCustomMetaTagsProperties()
        if kw.get('seo_custommetatags_override'):
            custommetatags = kw.get('seo_custommetatags', {})
            self.updateSEOCustomMetaTagsProperties(custommetatags)

    def getPropertyStopWords(self):
        """ Get property 'stop_words' from SEO Properties tool.
        """
        enc = getSiteEncoding(self.context)
        # self.gseo.stop_words return list of unicode objects,
        # and may contains stop words in different languages.
        # So we must return encoded strings.
        sw = map(lambda x:unicode.encode(x, enc), self.gseo.stop_words)
        return str(sw)

    def getPropertyFields(self):
        """ Get property 'fields' from SEO Properties tool.
        """
        # self.gseo.fields return list of unicode objects,
        # so *str* use as encoding function from unicode to latin-1 string.
        fields_id = map(str, self.gseo.fields)
        return str(fields_id)

    def __call__( self ):
        """ Perform the update seo properties and redirect if necessary,
            or render the page Call method.
        """
        context = aq_inner(self.context)
        request = self.request
        form = self.request.form
        submitted = form.get('form.submitted', False)
        if submitted:
            state = self.manageSEOProps(**form)
            if not state:
                state = _('seoproperties_saved', default=u'Content SEO properties have been saved.')
                context.plone_utils.addPortalMessage(state)
                kwargs = {'modification_date' : DateTime()} 
                context.plone_utils.contentEdit(context, **kwargs)
                return request.response.redirect(self.context.absolute_url())
            context.plone_utils.addPortalMessage(state, 'error')
        return self.template()


class VisibilityCheckerView( BrowserView ):
    """ This class contains methods that visibility checker.
    """

    def checkVisibilitySEOAction(self):
        """ Checks visibility 'SEO Properties' action for content
        """
        context = aq_inner(self.context)
        plone = queryMultiAdapter((self, self.request),name="plone_portal_state").portal()
        adapter = ISEOConfigletSchema(plone)
        return bool(self.context.portal_type in adapter.types_seo_enabled)




from cgi import escape
from DateTime import DateTime
from Acquisition import aq_inner

from zope.component import queryAdapter
from zope.component import queryMultiAdapter
from zope.component import getMultiAdapter
from plone.app.layout.viewlets.common import ViewletBase

from Products.CMFPlone.utils import safe_unicode, getSiteEncoding

from quintagroup.seoptimizer.util import SortedDict
from quintagroup.seoptimizer.interfaces import IMetaKeywords
from quintagroup.seoptimizer.interfaces import IMappingMetaTags
from quintagroup.seoptimizer.browser.seo_configlet import ISEOConfigletSchema

from Products.CMFPlone.PloneTool import *

class SEOTagsViewlet( ViewletBase ):
    """ Simple viewlet for custom title rendering.
    """

    def render(self):
        TEMPLATE = '<meta name="%s" content="%s"/>'
        enc = getSiteEncoding(self.context)
        sfuncd = lambda x, enc=enc:safe_unicode(x, enc)
        return u'\n'.join([TEMPLATE % tuple(map(sfuncd, (k,v))) \
                           for k,v in self.listMetaTags().items()])

    def listMetaTags(self):
        """Calculate list metatags"""

        result = SortedDict()
        pps = queryMultiAdapter((self.context, self.request), name="plone_portal_state")
        seo_global = queryAdapter(pps.portal(), ISEOConfigletSchema)
        seo_context = queryMultiAdapter((self.context, self.request), name='seo_context')

        use_all = seo_global.exposeDCMetaTags
        adapter = IMappingMetaTags(self.context, None)
        mapping_metadata = adapter and adapter.getMappingMetaTags() or SortedDict()

        if not use_all:
            metadata_names = mapping_metadata.has_key('DC.description') \
                             and {'DC.description': mapping_metadata['DC.description']} \
                             or SortedDict()
            if mapping_metadata.has_key('description'):
                metadata_names['description'] = mapping_metadata['description']
        else:
            metadata_names = mapping_metadata

        for key, accessor in metadata_names.items():
            if accessor == 'meta_keywords':
                # Render all the existing keywords for the current content type
                adapter = IMetaKeywords(self.context, None)
                if adapter is not None:
                    keywords = adapter.getMetaKeywords()
                    if keywords:
                        result['keywords'] = keywords
                continue

            if seo_context._seotags.has_key(accessor):
                value = seo_context._seotags.get(accessor, None)
            else:
                method = getattr(seo_context, accessor, None)
                if method is None:
                    method = getattr(aq_inner(self.context).aq_explicit, accessor, None)

                if not callable(method):
                    continue

                # Catch AttributeErrors raised by some AT applications
                try:
                    value = method()
                except AttributeError:
                    value = None

            if not value:
                # No data
                continue
            if accessor == 'Publisher' and value == 'No publisher':
                # No publisher is hardcoded (TODO: still?)
                continue
            if isinstance(value, (list, tuple)):
                # convert a list to a string
                value = ', '.join(value)

            # Special cases
            if accessor == 'Description' and not metadata_names.has_key('description'):
                result['description'] = value
            elif accessor == 'Subject' and not metadata_names.has_key('keywords'):
                result['keywords'] = value

            if accessor not in ('Description', 'Subject'):
                result[key] = value

        if use_all:
            created = self.context.CreationDate()

            try:
                effective = self.context.EffectiveDate()
                if effective == 'None':
                    effective = None
                if effective:
                    effective = DateTime(effective)
            except AttributeError:
                effective = None

            try:
                expires = self.context.ExpirationDate()
                if expires == 'None':
                    expires = None
                if expires:
                    expires = DateTime(expires)
            except AttributeError:
                expires = None

            # Filter out DWIMish artifacts on effective / expiration dates
            if effective is not None and \
               effective > FLOOR_DATE and \
               effective != created:
                eff_str = effective.Date()
            else:
                eff_str = ''

            if expires is not None and expires < CEILING_DATE:
                exp_str = expires.Date()
            else:
                exp_str = ''

            if exp_str or exp_str:
                result['DC.date.valid_range'] = '%s - %s' % (eff_str, exp_str)

        # add custom meta tags (added from qseo tab by user)
        # for given context and default from configlet
        custom_meta_tags = seo_context and seo_context['seo_customMetaTags'] or []
        for tag in custom_meta_tags:
            if tag['meta_content']:
                result[tag['meta_name']] = tag['meta_content']

        return result



class TitleCommentViewlet(ViewletBase):
    """ Simple viewlet for custom title rendering.
    """

    def update(self):
        self.portal_state = getMultiAdapter((self.context, self.request),
                                            name=u'plone_portal_state')
        self.context_state = getMultiAdapter((self.context, self.request),
                                             name=u'plone_context_state')
        self.seo_context = getMultiAdapter((self.context, self.request),
                                             name=u'seo_context')

        self.override_title = self.seo_context['has_seo_title']
        self.override_comments = self.seo_context['has_html_comment']

    def std_title(self):
        page_title = safe_unicode(self.context_state.object_title())
        portal_title = safe_unicode(self.portal_state.portal_title())
        if page_title == portal_title:
            return u"<title>%s</title>" % (escape(portal_title))
        else:
            return u"<title>%s &mdash; %s</title>" % (
                escape(safe_unicode(page_title)),
                escape(safe_unicode(portal_title)))

    def render(self):
        if not self.override_title:
            std_title = self.std_title()
            if not self.override_comments:
                return std_title
            else:
                qseo_comments = u"<!--%s-->" % safe_unicode(
                    self.seo_context["seo_html_comment"])
                return u"%s\n%s"%(std_title, qseo_comments)
        else:
            qseo_title = u"<title>%s</title>" % safe_unicode(
                self.seo_context["seo_title"])
            if not self.override_comments:
                return qseo_title
            else:
                qseo_comments = u"<!--%s-->" % safe_unicode(
                    self.seo_context["seo_html_comment"])
                return u"%s\n%s"%(qseo_title, qseo_comments)


class CustomScriptViewlet( ViewletBase ):
    """ Simple viewlet for custom script rendering.
    """
    def getCustomScript( self ):
        pps = queryMultiAdapter((self.context, self.request),
                                name="plone_portal_state")
        gseo = queryAdapter(pps.portal(), ISEOConfigletSchema)
        if gseo:
            return gseo.custom_script
        return ''

    def render( self ):
        return safe_unicode("""%s"""% self.getCustomScript())


class CanonicalUrlViewlet( ViewletBase ):
    """ Simple viewlet for canonical url link rendering.
    """
    def render( self ):
        seoc = getMultiAdapter((self.context, self.request), name=u'seo_context')
        if seoc['seo_canonical']:
            return """<link rel="canonical" href="%s" />""" % seoc['seo_canonical']
        return ""



from Products.CMFPlone.PloneTool import PloneTool

originalListMetaTags = PloneTool.listMetaTags

def qsListMetaTags(self, context):
    """ Custom listMetaTags method
    """
    from quintagroup.seoptimizer.browser.interfaces import IPloneSEOLayer
    if not IPloneSEOLayer.providedBy(self.REQUEST):
        return originalListMetaTags(self, context)
    return {}

def qsListMetaTagsOriginal(self, context):
    """ Returned original method listMetaTags
    """
    return originalListMetaTags(self, context)



from zope.interface import Interface
from plone.theme.interfaces import IDefaultPloneLayer

class IPloneSEOLayer(IDefaultPloneLayer):
    """Marker interface that defines a Zope 3 skin layer bound to a Skin
       Selection in portal_skins.
    """

class IValidateSEOKeywordsView(Interface):
    """ View for validating keywords on qSEO_properties_edit_form """

    def validateKeywords():
        """ 1. Get *text* parameter from request
            2. Parse *text* and validate each extracted keyword
               for appearing on the context page """



import re, commands
from xml.dom import Node

from zope.interface import implements
from Products.Five.browser import BrowserView

from Products.CMFPlone.utils import safe_unicode, getSiteEncoding
from Products.CMFCore.utils import getToolByName

from interfaces import IValidateSEOKeywordsView
from quintagroup.seoptimizer import SeoptimizerMessageFactory as _

class ValidateSEOKeywordsView(BrowserView):

    implements(IValidateSEOKeywordsView)

    def validateKeywords(self):
        """ see interface """
        text = self.request.get('text')
        ts = getToolByName(self.context, 'translation_service')
        # extract keywords from text
        enc = getSiteEncoding(self.context)
        if text.lower().strip():
            keywords = filter(None, map(lambda x: safe_unicode(x.strip(), enc),
                                         text.lower().strip().split('\n')))
        else:
            return ts.utranslate(domain='quintagroup.seoptimizer',
                                 msgid=_(u'Keywords list is empty!'),
                                 context=self.context)
        # request html page of context object
        url = '%s?without_metatag_keywords=1' % self.context.absolute_url()

        # extract words from url page using lynx browser (test page by 'url'
        # randered without metatag keywords)
        page_text = commands.getoutput('lynx --dump --nolist %s' % url).lower()
        if page_text and page_text != 'sh: lynx: command not found':
            page_text = safe_unicode(page_text, 'utf-8')
        else:
            return ts.utranslate(domain='quintagroup.seoptimizer',
                                 msgid=_(u'Could not find lynx browser!'),
                                 context=self.context)

        # check every keyword on appearing in body of html page
        result = []
        for keyword in keywords:
            keyword_on_page = unicode(len(re.findall(u'\\b%s\\b' % keyword, page_text, re.I|re.U)))
            result.append(' - '.join((keyword, keyword_on_page)))
        return ts.utranslate(domain='quintagroup.seoptimizer',
                             msgid=_(u'number_keywords',
                               default=u'Number of keywords at page:\n${result}',
                               mapping={'result':'\n'.join(result)}),
                             context=self.context)



from base import *

KWSTMPL = '.*(<meta\s+(?:(?:name="keywords"\s*)|(?:content="%s"\s*)){2}/>)'

class TestUsageKeywords(FunctionalTestCase):

    def afterSetUp(self):
        self.sp = self.portal.portal_properties.seo_properties
        self.pu = self.portal.plone_utils
        self.basic_auth = ':'.join((portal_owner,default_password))
        self.loginAsPortalOwner()
        #Preparation for functional testing
        self.my_doc = self.portal.invokeFactory('Document', id='my_doc')
        self.my_doc = self.portal['my_doc']

    def test_noDefaultKeywords(self):
        """No keywords added for the content by default"""
        metatags = self.pu.listMetaTags(self.my_doc)
        self.assert_('keywords' not in metatags)

    def testrender_SEOKeywords(self):
        """ """
        self.my_doc.setText('<p>foo bar</p>')
        self.my_doc.manage_addProperty('qSEO_keywords', [], 'lines')

        for seokws in [('foo',), ('foo', 'bar')]:
            self.my_doc._updateProperty('qSEO_keywords', seokws)
            html = str(self.publish(self.portal.id+'/my_doc', self.basic_auth))
            expect = ',\s*'.join(seokws)
            open('/tmp/testrender_SEOKeywords','w').write(html)
            self.assert_(re.match(KWSTMPL % expect, html, re.S|re.M),
                         "No '%s' keyword found" % str(seokws))

    def testbehave_NoSEOKeywordsOnlySubject(self):
        self.my_doc.setText('<p>local subject</p>')
        self.my_doc.setSubject('subject')
        html = str(self.publish(self.portal.id+'/my_doc', self.basic_auth))

        expect = "subject"
        self.assert_(re.match(KWSTMPL % expect, html, re.S|re.M),
                     "No '%s' keyword find" % expect)

    def testbehave_SEOKeywordsOverrideSubject(self):
        SEOKWS = ('local',)
        self.my_doc.setText('<p>local subject</p>')
        self.my_doc.setSubject('subject')
        self.my_doc.manage_addProperty('qSEO_keywords', SEOKWS, 'lines')
        html = str(self.publish(self.portal.id+'/my_doc', self.basic_auth))

        expect = ',\s*'.join(SEOKWS)
        self.assert_(re.match(KWSTMPL % expect, html, re.S|re.M),
                     "No '%s' keywords find" % SEOKWS)

    def testbehave_noSEOKeywordsNoSubject(self):
        """Nor seo keywords not subject added"""
        html = str(self.publish(self.portal.id+'/my_doc', self.basic_auth))
        self.assertFalse(re.match('.*(<meta\s[^\>]*name="keywords"[^\>]*>)',
                                  html, re.S|re.M), "'keyword' meta tag found")


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestUsageKeywords))
    return suite
